
// SerialLib.h

#ifndef _SERIALLIB_H_
#define _SERIALLIB_H_

//#define   DBG_COMPILE 1   //  define for use with boostc debugger-stops use of hardware

#include <system.h>

#define SERIAL_PRE 		0x0F
#define SERIAL_EOP 		0x04
#define SERIAL_ESC 		0x05
#define SERIAL_BUFSIZE 	32


#define B19_2AT20MEG   64       //19200	@ 4 meg  see base conversion spread sheet  #define B19_2AT4MEG 12
#define B19_2At4MEG    12

// beware of duplicate definitions 
volatile bit rcie 	@ PIE1.RCIE;
volatile bit rcif 	@ PIR1.RCIF;
volatile bit peie 	@ INTCON.PEIE;


// Function declarations
void serial_init(char brg);
void serial_init877(char brg);

void serial_printf(char value);
void serial_printf(const char* text);
void serial_print_lf();

void serial_print_hex( unsigned char 	number );
void serial_print_hex( unsigned short 	number );

void serial_print_dec(	unsigned	char 			number);
void serial_print_dec( 	signed 		char 			number);
void serial_print_dec(	unsigned 	short 			number);
void serial_print_dec(	signed 		short 			number);

void serial_print_dec(	char 						number, 	char positions);

void serial_print_bin(char number);

char serial_getch();
char serial_get_decimal();

// Function declarations for safe serial transmission
//void serial_safe_init();
//char serial_safe_putbyte(char data);
//char serial_safe_send();
//signed char serial_safe_receive();
//char serial_safe_getbyte(char index);

// Macros for simple functions (to reduce stack usage)
#define serial_peek()       (pir1 & 1 << RCIF)
#define serial_print_lf() 	serial_printf(0x0a); serial_printf(0x0d)


// ---------- begin inline --------
// interrupt on recieve
// best to have global interrupts off first and turn on when you are really ready
// also full duplex async 
inline void recieveInterruptOn ( void )
{


	
	// russ not checked, but assume ok for now 
    rcsta	= 10010000b; 	//full duplex asynchronous
    rcie 	= 1;			//enable RX interrupt
    peie 	= 1;			//enable peripheral interrupts (needed for RX)
	
	
}




// ========= begin command building =========

#define DATA_SEP 			","    		// 	seperates returned args
#define END_RESPONSE_SEP 	"\n"     	// 	signals end of response
#define ERROR_RESPONSE 		"!"    		// 	starts error messages

#define 	CMD_MAX        	10			// maximum number of characters in a command
// #define 	CMD_MAX_1		5

// add to command string
#define 	CMD_STATUS_ADD 	1
// not ready
#define 	CMD_STATUS_NR 	2

#define  	CHAR_LF  		0x0A 
#define		CHAR_CR  		0x0D 

// is this the backspace char? -- ref say it is but does not work or is the program ng
#define  	CHAR_BS  		0x08   
#define  	CHAR_DEL 		0x7F   

#define		STOP_CHAR 		'!'  // replaces the s and S I used to use 

extern  unsigned  	char 	Cmd[CMD_MAX];   
extern  unsigned  	char 	CmdPrior[CMD_MAX];

extern  unsigned	char 	stopFlag;

extern  unsigned 	char 	GotCmd;
extern  unsigned  	char  	cmdStatus;

void 	resetCmd( );

int 	parseCmdNbr(  const char* text );

void 	tryRecCmd( );

void   	buildCmd( unsigned char aNewChar );

void   	backCmd(  );


// ========= end command building  =========


#endif // _SERIALLIB_H_




